==> babel-preset-react-scope-style-0.0.1.tgz: package/esm/utils/index.js <==
"use strict";

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var t = require('@babel/types');

var findUp = require('find-up');

var fs = require('fs');

var path = require('path');

var template = require('@babel/template').default;

var ScopeName = 'scope-style';
var ClassNames = 'classNames';
var LibraryClassNames = 'classnames';

function formatDate(date, fmt) {
  var o = {
    'M+': date.getMonth() + 1,
    'd+': date.getDate(),
    'h+': date.getHours(),
    'm+': date.getMinutes(),
    's+': date.getSeconds(),
    'q+': Math.floor((date.getMonth() + 3) / 3),
    S: date.getMilliseconds()
  };
  if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
  Object.keys(o).forEach(function (k) {
    if (new RegExp('(' + k + ')').test(fmt)) {
      fmt = fmt.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ('00' + o[k]).substr(('' + o[k]).length));
    }
  });
  return fmt;
}

;

function fileExists(path) {
  try {
    return !fs.accessSync(path, fs.F_OK);
  } catch (e) {
    return false;
  }
}

function getImportDeclarations(path) {
  var program = path.isProgram() ? path : path.findParent(function (p) {
    return p.isProgram();
  });
  return program.node.body.filter(function (node) {
    return t.isImportDeclaration(node);
  });
}

function isImportLibrary(path) {
  var libraryName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : LibraryName;
  var declaration = getImportDeclarations(path).find(function (node) {
    return node.source.value === libraryName;
  }); // path.traverse({
  //   ImportDeclaration(path) {
  //     if (path.node.source.value === libraryName) declaration = path.node;
  //     path.stop();
  //   },
  // });

  return declaration;
}

function arr2Expression(arr, parent) {
  var temp = '';
  var vars = {};
  arr.forEach(function (v, i) {
    var expr = var2Expression(v, arr);
    if (!expr) return;
    var key = "$".concat(i);
    temp += (temp ? ', ' : '') + key;
    vars[key] = expr;
  });
  return template("[".concat(temp, "]"))(vars);
}

function obj2Expression(obj, parent) {
  var props = Object.keys(obj).map(function (k) {
    var v = obj[k];
    var expr = var2Expression(v, obj);
    if (!expr) return;
    return t.objectProperty(t.identifier(k), expr);
  }).filter(Boolean);
  return t.objectExpression(props);
}

function var2Expression(v, parent) {
  if (t.isNode(v)) return v;
  if (v === undefined) return;
  if (Array.isArray(v)) return arr2Expression(v, parent);

  switch (_typeof(v)) {
    case 'string':
      return t.stringLiteral(v);

    case 'boolean':
      return t.booleanLiteral(v);

    case 'number':
      return t.numericLiteral(v);

    case 'object':
      if (v === null) return t.nullLiteral();
      if (v instanceof RegExp) return t.regExpLiteral(v.source, v.flags);
      if (v instanceof Date) return template('new Date(TIME)')({
        TIME: t.numericLiteral(v.getTime())
      });
      if (v instanceof Function) return template(v.toString())();
      return obj2Expression(v, parent);

    default:
      return t.identifier('undefined');
  }
}

function memberExpr2Str(expr) {
  var objStr;
  var object = expr.object;
  if (!object) return String(expr.value);

  switch (expr.object.type) {
    case 'MemberExpression':
    case 'JSXMemberExpression':
      objStr = memberExpr2Str(expr.object);
      break;

    default:
      objStr = expr2str(expr.object);
  }

  var propIsMember = expr.property.type === 'MemberExpression';
  var propStr = expr2str(expr.property);
  return objStr + (objStr && !propIsMember ? '.' : '') + (propIsMember ? "[".concat(propStr, "]") : propStr);
}

function expr2str(expr) {
  if (!expr) return '';
  if (typeof expr === 'string') return expr; // if (expr.extra) return expr.extra.raw;

  switch (expr.type) {
    case 'JSXExpressionContainer':
      return expr2str(expr.expression);

    case 'MemberExpression':
    case 'JSXMemberExpression':
      return memberExpr2Str(expr);

    case 'Identifier':
    case 'JSXIdentifier':
      return expr.name;

    case 'JSXNamespacedName':
      return "".concat(expr.namespace.name, ":").concat(expr.name.name);

    case 'ThisExpression':
      return 'this';

    case 'NumericLiteral':
    case 'BooleanLiteral':
    case 'StringLiteral':
      return expr.value;

    case 'NullLiteral':
      return 'null';

    case 'RegExpLiteral':
      return "/".concat(expr.pattern, "/").concat(expr.flags);

    case 'SpreadElement':
      return "...".concat(expr2str(expr.argument));

    case 'BinaryExpression':
      return "".concat(expr2str(expr.left), " ").concat(expr.operator, " ").concat(expr2str(expr.right));

    case 'UpdateExpression':
    case 'UnaryExpression':
      return "".concat(expr.prefix ? expr.operator : '').concat(expr2str(expr.argument)).concat(!expr.prefix ? expr.operator : '');

    case 'ConditionalExpression':
      return "".concat(expr2str(expr.test), " ? ").concat(expr2str(expr.consequent), " : ").concat(expr2str(expr.alternate));

    case 'CallExpression':
      return "".concat(expr2str(expr.callee), "(").concat(expr.arguments.map(function (a) {
        return expr2str(a);
      }).join(','), ")");

    case 'NewExpression':
      return "new ".concat(expr2str(expr.callee), "(").concat(expr.arguments.map(function (a) {
        return expr2str(a);
      }).join(','), ")");

    case 'VariableDeclarator':
      return "".concat(expr.id).concat(expr.init ? " = ".concat(expr2str(expr.init)) : '');

    case 'VariableDeclaration':
      return "".concat(expr.kind, " ").concat(expr.declarations.map(function (d) {
        return expr2str(d);
      }), ";");

    case 'BlockStatement':
      return "{".concat(expr2str(expr.body), "}");

    case 'TemplateLiteral':
      return temp2var(expr);

    case 'TaggedTemplateExpression':
      return "".concat(expr2str(expr2str(expr.tag))).concat(expr2str(expr.quasi));

    case 'FunctionExpression':
      return "function ".concat(expr2str(expr.id), "(").concat(expr.params.map(function (a) {
        return expr2str(a);
      }).join(','), ")").concat(expr2str(expr.body));

    case 'AssignmentPattern':
      return "".concat(expr2str(expr.left), " = ").concat(expr2str(expr.right));

    case 'ArrayExpression':
    case 'ArrayPattern':
      return "[".concat(expr.elements.map(function (v) {
        return expr2str(v);
      }).join(', '), "]");

    case 'ObjectProperty':
      return "".concat(expr.computed ? "[".concat(expr2str(expr.key), "]") : expr2str(expr.key), ": ").concat(expr2str(expr.value));

    case 'ObjectMethod':
      // eslint-disable-next-line
      return "".concat(expr.kind !== 'method' ? "".concat(expr.kind, " ") : '').concat(expr2str(expr.key), "(").concat(expr.params.map(function (a) {
        return expr2str(a);
      }).join(', '), ")").concat(expr2str(expr.body));

    case 'ObjectPattern':
    case 'ObjectExpression':
      return "{".concat(expr.properties.map(function (v) {
        return expr2str(v);
      }).join(', '), "}");

    default:
      return '';
  }
}

function isFunction(fn) {
  return typeof fn === 'function';
}

function isRequired(path, name) {
  var ctx = {
    imported: false
  };
  var names = Array.isArray(name) ? _toConsumableArray(name) : [name];
  if (!names.length) return true;
  path.traverse({
    ImportDeclaration: function ImportDeclaration(path) {
      var _path$node$source = path.node.source,
          type = _path$node$source.type,
          value = _path$node$source.value;
      if (type !== 'StringLiteral') return;
      var idx = names.indexOf(value);
      if (idx < 0) return;
      names.splice(idx, 1);

      if (!names.length) {
        this.imported = true;
        path.stop();
      }
    }
  }, ctx);
  return ctx.imported;
}

function isReactComponent(path) {
  return isRequired(path, 'react');
}

function isImportSpecifier(path, specifierName, declaration) {
  var libraryName = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : LibraryName;
  var declarations;

  if (!declaration) {
    if (libraryName) declaration = isImportLibrary(path, libraryName);else declarations = getImportDeclarations(path);
  }

  if (declaration) declarations = [declaration];
  var ret;
  declarations && declarations.some(function (item) {
    return ret = item.specifiers.find(function (v) {
      return v.local.name === specifierName;
    });
  });
  return ret;
}

function importSpecifier(path, specifierName) {
  var libraryName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : LibraryName;
  var declaration = isImportLibrary(path, libraryName);

  var _specifierName$split = specifierName.split(','),
      _specifierName$split2 = _slicedToArray(_specifierName$split, 2),
      local = _specifierName$split2[0],
      _specifierName$split3 = _specifierName$split2[1],
      imported = _specifierName$split3 === void 0 ? specifierName : _specifierName$split3;

  var specifier = imported === 'default' ? t.importDefaultSpecifier(t.identifier(local)) : t.importSpecifier(t.identifier(local), t.identifier(imported));

  if (declaration) {
    if (!isImportSpecifier(path, specifierName, declaration)) {
      declaration.specifiers.push(specifier);
    }
  } else {
    var program = path.isProgram() ? path : path.findParent(function (p) {
      return p.isProgram();
    });
    program.unshiftContainer('body', t.importDeclaration([specifier], t.stringLiteral(libraryName)));
  }

  return specifier;
}

function importDefaultSpecifier(path, specifierName) {
  var libraryName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : LibraryName;
  return importSpecifier(path, "".concat(specifierName, ",default"), libraryName);
}

module.exports = {
  ScopeName: ScopeName,
  ClassNames: ClassNames,
  LibraryClassNames: LibraryClassNames,
  fileExists: fileExists,
  isFunction: isFunction,
  isRequired: isRequired,
  isReactComponent: isReactComponent,
  isImportLibrary: isImportLibrary,
  var2Expression: var2Expression,
  arr2Expression: arr2Expression,
  obj2Expression: obj2Expression,
  memberExpr2Str: memberExpr2Str,
  expr2str: expr2str,
  isImportSpecifier: isImportSpecifier,
  importSpecifier: importSpecifier,
  importDefaultSpecifier: importDefaultSpecifier
};

==> babel-preset-react-scope-style-0.0.2.tgz: package/esm/utils/index.js <==
"use strict";

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var t = require('@babel/types');

var fs = require('fs');

var template = require('@babel/template').default;

var ScopeName = 'scope-style';
var ClassNames = 'classNames';
var LibraryClassNames = 'classnames';

function fileExists(path) {
  try {
    return !fs.accessSync(path, fs.F_OK);
  } catch (e) {
    return false;
  }
}

function getImportDeclarations(path) {
  var program = path.isProgram() ? path : path.findParent(function (p) {
    return p.isProgram();
  });
  return program.node.body.filter(function (node) {
    return t.isImportDeclaration(node);
  });
}

function isImportLibrary(path, libraryName) {
  var declaration = getImportDeclarations(path).find(function (node) {
    return node.source.value === libraryName;
  });
  return declaration;
}

function arr2Expression(arr, parent) {
  var temp = '';
  var vars = {};
  arr.forEach(function (v, i) {
    // eslint-disable-next-line no-use-before-define
    var expr = var2Expression(v, arr);
    if (!expr) return;
    var key = "$".concat(i);
    temp += (temp ? ', ' : '') + key;
    vars[key] = expr;
  });
  return template("[".concat(temp, "]"))(vars);
}

function obj2Expression(obj, parent) {
  var props = Object.keys(obj).map(function (k) {
    var v = obj[k]; // eslint-disable-next-line no-use-before-define

    var expr = var2Expression(v, obj);
    if (!expr) return;
    return t.objectProperty(t.identifier(k), expr);
  }).filter(Boolean);
  return t.objectExpression(props);
}

function var2Expression(v, parent) {
  if (t.isNode(v)) return v;
  if (v === undefined) return;
  if (Array.isArray(v)) return arr2Expression(v, parent);

  switch (_typeof(v)) {
    case 'string':
      return t.stringLiteral(v);

    case 'boolean':
      return t.booleanLiteral(v);

    case 'number':
      return t.numericLiteral(v);

    case 'object':
      if (v === null) return t.nullLiteral();
      if (v instanceof RegExp) return t.regExpLiteral(v.source, v.flags);
      if (v instanceof Date) return template('new Date(TIME)')({
        TIME: t.numericLiteral(v.getTime())
      });
      if (v instanceof Function) return template(v.toString())();
      return obj2Expression(v, parent);

    default:
      return t.identifier('undefined');
  }
}

function memberExpr2Str(expr) {
  var objStr;
  var object = expr.object;
  if (!object) return String(expr.value);

  switch (expr.object.type) {
    case 'MemberExpression':
    case 'JSXMemberExpression':
      objStr = memberExpr2Str(expr.object);
      break;

    default:
      // eslint-disable-next-line no-use-before-define
      objStr = expr2str(expr.object);
  }

  var propIsMember = expr.property.type === 'MemberExpression'; // eslint-disable-next-line no-use-before-define

  var propStr = expr2str(expr.property);
  return objStr + (objStr && !propIsMember ? '.' : '') + (propIsMember ? "[".concat(propStr, "]") : propStr);
}

function expr2str(expr) {
  if (!expr) return '';
  if (typeof expr === 'string') return expr; // if (expr.extra) return expr.extra.raw;

  switch (expr.type) {
    case 'JSXExpressionContainer':
      return expr2str(expr.expression);

    case 'MemberExpression':
    case 'JSXMemberExpression':
      return memberExpr2Str(expr);

    case 'Identifier':
    case 'JSXIdentifier':
      return expr.name;

    case 'JSXNamespacedName':
      return "".concat(expr.namespace.name, ":").concat(expr.name.name);

    case 'ThisExpression':
      return 'this';

    case 'NumericLiteral':
    case 'BooleanLiteral':
    case 'StringLiteral':
      return expr.value;

    case 'NullLiteral':
      return 'null';

    case 'RegExpLiteral':
      return "/".concat(expr.pattern, "/").concat(expr.flags);

    case 'SpreadElement':
      return "...".concat(expr2str(expr.argument));

    case 'BinaryExpression':
      return "".concat(expr2str(expr.left), " ").concat(expr.operator, " ").concat(expr2str(expr.right));

    case 'UpdateExpression':
    case 'UnaryExpression':
      return "".concat(expr.prefix ? expr.operator : '').concat(expr2str(expr.argument)).concat(!expr.prefix ? expr.operator : '');

    case 'ConditionalExpression':
      return "".concat(expr2str(expr.test), " ? ").concat(expr2str(expr.consequent), " : ").concat(expr2str(expr.alternate));

    case 'CallExpression':
      return "".concat(expr2str(expr.callee), "(").concat(expr.arguments.map(function (a) {
        return expr2str(a);
      }).join(','), ")");

    case 'NewExpression':
      return "new ".concat(expr2str(expr.callee), "(").concat(expr.arguments.map(function (a) {
        return expr2str(a);
      }).join(','), ")");

    case 'VariableDeclarator':
      return "".concat(expr.id).concat(expr.init ? " = ".concat(expr2str(expr.init)) : '');

    case 'VariableDeclaration':
      return "".concat(expr.kind, " ").concat(expr.declarations.map(function (d) {
        return expr2str(d);
      }), ";");

    case 'BlockStatement':
      return "{".concat(expr2str(expr.body), "}");

    case 'TemplateLiteral':
      // eslint-disable-next-line no-use-before-define
      return temp2var(expr);

    case 'TaggedTemplateExpression':
      return "".concat(expr2str(expr2str(expr.tag))).concat(expr2str(expr.quasi));

    case 'FunctionExpression':
      return "function ".concat(expr2str(expr.id), "(").concat(expr.params.map(function (a) {
        return expr2str(a);
      }).join(','), ")").concat(expr2str(expr.body));

    case 'AssignmentPattern':
      return "".concat(expr2str(expr.left), " = ").concat(expr2str(expr.right));

    case 'ArrayExpression':
    case 'ArrayPattern':
      return "[".concat(expr.elements.map(function (v) {
        return expr2str(v);
      }).join(', '), "]");

    case 'ObjectProperty':
      return "".concat(expr.computed ? "[".concat(expr2str(expr.key), "]") : expr2str(expr.key), ": ").concat(expr2str(expr.value));

    case 'ObjectMethod':
      // eslint-disable-next-line
      return "".concat(expr.kind !== 'method' ? "".concat(expr.kind, " ") : '').concat(expr2str(expr.key), "(").concat(expr.params.map(function (a) {
        return expr2str(a);
      }).join(', '), ")").concat(expr2str(expr.body));

    case 'ObjectPattern':
    case 'ObjectExpression':
      return "{".concat(expr.properties.map(function (v) {
        return expr2str(v);
      }).join(', '), "}");

    default:
      return '';
  }
}

function temp2var(expr) {
  var arr = [].concat(_toConsumableArray(expr.expressions), _toConsumableArray(expr.quasis)).sort(function (a, b) {
    return a.start - b.start;
  });
  var ret = '';
  arr.forEach(function (v) {
    if (v.type === 'TemplateElement') ret += v.value.raw;else ret += '${' + expr2str(v) + '}';
  });
  return '`' + ret + '`';
}

function isFunction(fn) {
  return typeof fn === 'function';
}

function isRequired(path, name) {
  var ctx = {
    imported: false
  };
  var names = Array.isArray(name) ? _toConsumableArray(name) : [name];
  if (!names.length) return true;
  path.traverse({
    ImportDeclaration: function ImportDeclaration(path) {
      var _path$node$source = path.node.source,
          type = _path$node$source.type,
          value = _path$node$source.value;
      if (type !== 'StringLiteral') return;
      var idx = names.indexOf(value);
      if (idx < 0) return;
      names.splice(idx, 1);

      if (!names.length) {
        this.imported = true;
        path.stop();
      }
    }
  }, ctx);
  return ctx.imported;
}

function isReactComponent(path) {
  return isRequired(path, 'react');
}

function isImportSpecifier(path, specifierName, declaration, libraryName) {
  var declarations;

  if (!declaration) {
    if (libraryName) declaration = isImportLibrary(path, libraryName);else declarations = getImportDeclarations(path);
  }

  if (declaration) declarations = [declaration];
  var ret;
  declarations && declarations.some(function (item) {
    return ret = item.specifiers.find(function (v) {
      return v.local.name === specifierName;
    });
  });
  return ret;
}

function importSpecifier(path, specifierName, libraryName) {
  var declaration = isImportLibrary(path, libraryName);

  var _specifierName$split = specifierName.split(','),
      _specifierName$split2 = _slicedToArray(_specifierName$split, 2),
      local = _specifierName$split2[0],
      _specifierName$split3 = _specifierName$split2[1],
      imported = _specifierName$split3 === void 0 ? specifierName : _specifierName$split3;

  var specifier = imported === 'default' ? t.importDefaultSpecifier(t.identifier(local)) : t.importSpecifier(t.identifier(local), t.identifier(imported));

  if (declaration) {
    if (!isImportSpecifier(path, specifierName, declaration)) {
      declaration.specifiers.push(specifier);
    }
  } else {
    var program = path.isProgram() ? path : path.findParent(function (p) {
      return p.isProgram();
    });
    program.unshiftContainer('body', t.importDeclaration([specifier], t.stringLiteral(libraryName)));
  }

  return specifier;
}

function importDefaultSpecifier(path, specifierName, libraryName) {
  return importSpecifier(path, "".concat(specifierName, ",default"), libraryName);
}

module.exports = {
  ScopeName: ScopeName,
  ClassNames: ClassNames,
  LibraryClassNames: LibraryClassNames,
  fileExists: fileExists,
  isFunction: isFunction,
  isRequired: isRequired,
  isReactComponent: isReactComponent,
  isImportLibrary: isImportLibrary,
  var2Expression: var2Expression,
  arr2Expression: arr2Expression,
  obj2Expression: obj2Expression,
  memberExpr2Str: memberExpr2Str,
  expr2str: expr2str,
  isImportSpecifier: isImportSpecifier,
  importSpecifier: importSpecifier,
  importDefaultSpecifier: importDefaultSpecifier
};
